import os
import io
from pathlib import Path
from openai import OpenAI
from pydub import AudioSegment

TEXT_SEND_LIMIT = 4096  # Constant for the text limit


def split_text(text, limit=TEXT_SEND_LIMIT):
    words = text.split()
    chunks = []
    current_chunk = words[0]

    for word in words[1:]:
        if len(current_chunk) + len(word) + 1 <= limit:
            current_chunk += " " + word
        else:
            chunks.append(current_chunk)
            current_chunk = word
    chunks.append(current_chunk)

    return chunks


def process_article(text, filename, model, voice):
    client = OpenAI()
    chunks = split_text(text)

    output_path = Path(filename)
    output_format = output_path.suffix.lstrip(".")

    combined_audio = AudioSegment.empty()
    success = True

    for i, chunk in enumerate(chunks, start=1):
        try:
            response = client.audio.speech.create(model=model, voice=voice, input=chunk)
            part_audio = AudioSegment.from_file(
                io.BytesIO(response.content), format=output_format
            )
            combined_audio += part_audio
        except Exception as e:
            print(f"An error occurred for part {i}: {e}")
            import traceback

            traceback.print_exception(type(e), e, e.__traceback__)
            if "429" in str(e):
                print("Quota exceeded. Stopping further requests.")
                success = False
                break

    if success and not combined_audio.empty():
        combined_audio.export(output_path, format=output_format)
        print(f"Combined audio saved to {output_path}")
    else:
        print("No audio generated due to errors.")
        if output_path.exists():
            output_path.unlink()  # Ensure no partial files are left



from click.testing import CliRunner
from article_to_podcast.cli import cli
from article_to_podcast.main import TEXT_SEND_LIMIT, split_text
from article_to_podcast.article_fetcher import get_article_content
from pathlib import Path
import pytest

ARTICLE_URL = "https://blog.kubetools.io/kopylot-an-ai-powered-kubernetes-assistant-for-devops-developers/"


def test_split_text():
    text = "This is a test text. " * 300  # Creating a long text to ensure it gets split
    chunks = split_text(text)
    assert len(chunks) > 1  # Ensure that the text is split into more than one chunk
    for chunk in chunks:
        assert (
            len(chunk) <= TEXT_SEND_LIMIT
        )  # Ensure that each chunk is within the limit


def test_get_article_content():
    text, title = get_article_content(ARTICLE_URL)
    assert (
        "KoPylot\xa0is a cloud-native application performance monitoring (APM) solution that runs on Kubernetes"
        in text
    )
    assert "KoPylot" in title  # Checking a part of the title to ensure it's correct


def test_process_article():
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "--url",
            ARTICLE_URL,
            "--directory",
            "/tmp",
            "--audio-format",
            "mp3",
            "--model",
            "tts-1",
            "--voice",
            "alloy",
        ],
        catch_exceptions=False,  # Allow exceptions to propagate
    )

    if result.exception:
        print(f"Exception: {result.exception}")
        import traceback

        traceback.print_exception(
            type(result.exception), result.exception, result.exception.__traceback__
        )

    # Ensure test fails if there's an exception
    assert result.exception is None or "429" in str(result.exception)

    output_audio_paths = list(Path("/tmp").glob("*.mp3"))
    if output_audio_paths:
        for output_audio_path in output_audio_paths:
            assert not output_audio_path.exists()  # Ensure no partial files are saved
            # Clean up
            if output_audio_path.exists():
                output_audio_path.unlink()



import click
from .main import process_article
from .article_fetcher import get_article_content
from pathlib import Path
import re


def format_filename(title, format):
    # Replace special characters with dashes and convert to lowercase
    formatted_title = re.sub(r"\W+", "-", title).strip("-").lower()
    return f"{formatted_title}.{format}"


@click.command()
@click.option(
    "--url", type=str, required=True, help="URL of the article to be fetched."
)
@click.option(
    "--directory",
    type=click.Path(exists=True, file_okay=False, writable=True),
    required=True,
    help="Directory where the output audio file will be saved. The filename will be derived from the article title.",
)
@click.option(
    "--audio-format",
    type=click.Choice(["mp3", "opus", "aac", "flac", "pcm"]),
    default="mp3",
    help="The audio format for the output file. Default is mp3.",
)
@click.option(
    "--model",
    type=click.Choice(["tts-1", "tts-1-hd"]),
    default="tts-1",
    help="The model to be used for text-to-speech conversion.",
)
@click.option(
    "--voice",
    type=click.Choice(["alloy", "echo", "fable", "onyx", "nova", "shimmer"]),
    default="alloy",
    help="""
    The voice to be used for the text-to-speech conversion. Voice options:
    alloy:   A balanced and neutral voice.
    echo:    A more dynamic and engaging voice.
    fable:   A narrative and storytelling voice.
    onyx:    A deep and resonant voice.
    nova:    A bright and energetic voice.
    shimmer: A soft and soothing voice.
    Experiment with different voices to find one that matches your desired tone and audience. The current voices are optimized for English.
    """,
)
def cli(url, directory, audio_format, model, voice):
    text, title = get_article_content(url)
    filename = format_filename(title, audio_format)
    filepath = Path(directory) / filename
    process_article(text, str(filepath), model, voice)


if __name__ == "__main__":
    cli()



import requests
from bs4 import BeautifulSoup
from readability import Document


def get_article_content(url):
    response = requests.get(url)
    doc = Document(response.text)
    soup = BeautifulSoup(doc.summary(), "html.parser")
    title = doc.title()
    text = soup.get_text()
    return text, title
